#!/usr/bin/env ruby

ARG_NUM = 4
TYPES_SYM = ['R_CHAR', 'R_INT', 'R_FLOAT', 'R_PTR', 'R_VOID']
TYPES_NAME = ['char', 'int', 'float', 'void*', 'void']
TYPES_MEM = ['c', 'i', 'f', 'p']
R_VOID = TYPES_SYM.size-1

print %Q(/* this is generated code */

switch(arg_num) {
)

def mk_dispatch_rec(arg_num, rtype, args)
  if (args.size == arg_num)
    fp = TYPES_NAME[rtype] + " (*)("
    fp += args.collect {|a| TYPES_NAME[a]}.join(',') + ")"

    ret = ""
    if (rtype != R_VOID)
      ret = "ret->#{TYPES_MEM[rtype]}="
    end

    as = Array.new
    args.each_with_index do |a, i|
      as << "args[#{i}].#{TYPES_MEM[a]}"
    end

    print "#{ret}((#{fp})fp)(#{as.join(',')});\n"
  else
    print "switch (types[#{args.size}]) {\n"
    TYPES_SYM.each_index do |i|
      break if (i == R_VOID)
      print "case #{TYPES_SYM[i]}:\n"
      mk_dispatch_rec(arg_num, rtype, args.dup.push(i))
      print "break;\n"
    end
    print "}\n"
  end
end

def mk_dispatch(arg_num)
  print "case #{arg_num}:\n"
  print "switch (rtype) {\n"
  TYPES_SYM.each_index do |i|
    print "case #{TYPES_SYM[i]}:\n"
    mk_dispatch_rec(arg_num, i, [])
    print "break;\n"
  end
  print "} break;\n"
end

0.upto(ARG_NUM) do |i|
  mk_dispatch(i)
end

print %Q(default:
  fprintf(stderr, "invalid arguments\\n");
  return 1;
}

    return 0;
)
