/**
 * $Id$
 *
 * Copyright (C) shinichiro.h <hamaji@nii.ac.jp>
 *  http://shinh.skr.jp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef reflection_h_
#define reflection_h_

#include <stdio.h>

#include "hashtab.h"

typedef union {
    char c;
    int i;
    float f;
    double d;
    void* p;
} reflection_value_t;

typedef enum {
    R_VOID, R_CHAR, R_INT, R_FLOAT, R_PTR,
#ifdef HAVE_FFI
    R_DOUBLE
#endif
} reflection_type_t;

typedef struct {
    char* name;
    int fp;
} reflection_sym_t;

typedef struct {
    htab_t syms;
    int sym_num;
    char** files;
    int file_num;
} reflection_t;

reflection_t* reflection_init(const char* exe, int tbl_size);
int reflection_quit(reflection_t* ref);

int reflection_dispatch(
    reflection_t* ref, const char* name, reflection_type_t rtype,
    int arg_num, reflection_value_t* args, reflection_type_t* types,
    reflection_value_t* ret);

int reflection_dlopen(reflection_t* ref, const char* file, const char* func);

void reflection_dump_symbols(reflection_t* ret);

#endif // ! reflection_h_

